@@ -56,23 +56,8 @@
  */
 #define RSVG_DEFAULT_DPI_X 90.0
 #define RSVG_DEFAULT_DPI_Y 90.0
-static double internal_dpi_x = RSVG_DEFAULT_DPI_X;
-static double internal_dpi_y = RSVG_DEFAULT_DPI_Y;
-
-static void
-rsvg_ctx_free_helper (gpointer key, gpointer value, gpointer user_data)
-{
-	xmlEntityPtr entval = (xmlEntityPtr)value;
-	
-	/* key == entval->name, so it's implicitly freed below */
-	
-	g_free ((char *) entval->name);
-	g_free ((char *) entval->ExternalID);
-	g_free ((char *) entval->SystemID);
-	xmlFree (entval->content);
-	xmlFree (entval->orig);
-	g_free (entval);
-}
+double rsvg_internal_dpi_x = RSVG_DEFAULT_DPI_X;
+double rsvg_internal_dpi_y = RSVG_DEFAULT_DPI_Y;
 
 typedef struct _RsvgSaxHandlerDefs {
 	RsvgSaxHandler super;
@@ -641,12 +626,6 @@ rsvg_characters (void *data, const xmlChar *ch, int len)
 #warning "libxml version less than 2.6.22. XML entities won't work"
 #endif
 
-#if LIBXML_VERSION >= 20621
-#define RSVG_ENABLE_ENTITIES
-#elif defined(__GNUC__)
-#warning "libxml version less than 2.6.22. XML entities won't work"
-#endif
-
 static xmlEntityPtr
 rsvg_get_entity (void *data, const xmlChar *name)
 {
@@ -704,7 +683,7 @@ rsvg_error_cb (void *data, const char *msg, ...)
 static xmlSAXHandler rsvgSAXHandlerStruct;
 static gboolean rsvgSAXHandlerStructInited = FALSE;
 
-static void rsvg_SAX_handler_struct_init()
+void rsvg_SAX_handler_struct_init (void)
 {
 	if(!rsvgSAXHandlerStructInited) 
 		{
@@ -743,10 +722,11 @@ rsvg_get_base_uri_from_filename(const gchar * file_name)
 }
 
 /**
- * Set the base URI for this SVG
- *
+ * rsvg_handle_set_base_uri
  * @handle: A #RsvgHandle
- * @base_uri: 
+ * @base_uri: The base uri
+ *
+ * Set the base URI for this SVG
  *
  * Since: 2.9 (really present in 2.8 as well)
  */
@@ -764,9 +744,11 @@ void rsvg_handle_set_base_uri (RsvgHandle *handle,
 }
 
 /**
- * Gets the base uri for this RsvgHandle.
+ * rsvg_handle_get_base_uri:
  * @handle: A #RsvgHandle
  *
+ * Gets the base uri for this #RsvgHandle.
+ *
  * Returns: the base uri, possibly null
  * Since: 2.9 (really present in 2.8 as well)
  */
@@ -818,7 +800,7 @@ rsvg_handle_write_impl (RsvgHandle    *handle,
 		g_propagate_error (error, real_error);
 		return FALSE;
 		}*/
-  return TRUE;
+	return TRUE;
 }
 
 static gboolean
@@ -881,29 +863,6 @@ rsvg_drawing_ctx_free (RsvgDrawingCtx *handle)
 	g_free (handle);
 }
 
-static void
-rsvg_handle_free_impl (RsvgHandle *handle)
-{
-	g_hash_table_foreach (handle->entities, rsvg_ctx_free_helper, NULL);
-	g_hash_table_destroy (handle->entities);
-	rsvg_defs_free (handle->defs);
-	g_hash_table_destroy (handle->css_props);
-	
-	if (handle->user_data_destroy)
-		(* handle->user_data_destroy) (handle->user_data);
-
-	if (handle->title)
-		g_string_free (handle->title, TRUE);
-	if (handle->desc)
-		g_string_free (handle->desc, TRUE);
-	if (handle->metadata)
-		g_string_free (handle->metadata, TRUE);
-	if (handle->base_uri)
-		g_free (handle->base_uri);
-
-	g_free (handle);
-}
-
 /**
  * rsvg_handle_get_metadata:
  * @handle: An #RsvgHandle
@@ -967,43 +926,6 @@ G_CONST_RETURN char *rsvg_handle_get_desc (RsvgHandle *handle)
 		return NULL;
 }
 
-/**
- * rsvg_handle_new:
- *
- * Returns a new rsvg handle.  Must be freed with @rsvg_handle_free.  This
- * handle can be used for dynamically loading an image.  You need to feed it
- * data using @rsvg_handle_write, then call @rsvg_handle_close when done.  No
- * more than one image can be loaded with one handle.
- *
- * Returns: A new #RsvgHandle
- **/
-RsvgHandle *
-rsvg_handle_new (void)
-{
-	RsvgHandle *handle;
-	
-	handle = g_new0 (RsvgHandle, 1);
-
-	handle->defs = rsvg_defs_new ();
-	handle->handler_nest = 0;
-	handle->entities = g_hash_table_new (g_str_hash, g_str_equal);
-	handle->dpi_x = internal_dpi_x;
-	handle->dpi_y = internal_dpi_y;
-	
-	handle->css_props = g_hash_table_new_full (g_str_hash, g_str_equal,
-											   g_free, g_free);
-	rsvg_SAX_handler_struct_init();
-	
-	handle->ctxt = NULL;
-	handle->currentnode = NULL;
-	handle->treebase = NULL;
-
-	handle->finished = 0;
-	handle->first_write = TRUE;
-
-	return handle;
-}
-
 typedef struct {
 	RsvgRender super;
 	RsvgBbox bbox;
@@ -1104,6 +1026,7 @@ _rsvg_find_bbox (RsvgHandle *handle)
 	RsvgDrawingCtx * ctx = g_new(RsvgDrawingCtx, 1);
 	RsvgBbox output;
 	RsvgBboxRender * render = rsvg_bbox_render_new();
+	ctx->drawsub_stack = NULL;
 	ctx->render = (RsvgRender *)render;
 	
 	ctx->state = NULL;
@@ -1129,21 +1052,28 @@ _rsvg_find_bbox (RsvgHandle *handle)
 	return output;
 }
 
+/**
+ * rsvg_handle_get_dimensions
+ * @handle: A #RsvgHandle
+ * @dimension_data: A place to store the SVG's size
+ *
+ * Get the SVG's size. Do not call from within the size_func callback, because an infinite loop will occur.
+ *
+ * Since: 2.14
+ */
 void
-rsvg_handle_get_dimensions(RsvgHandle * handle, RsvgDimensionData * output)
+rsvg_handle_get_dimensions(RsvgHandle * handle, RsvgDimensionData * dimension_data)
 {
 	RsvgNodeSvg * sself;
 	RsvgBbox bbox;
 
-	g_return_if_fail(output);
-	memset(output, 0, sizeof(RsvgDimensionData));
+	g_return_if_fail(dimension_data);
+	memset(dimension_data, 0, sizeof(RsvgDimensionData));
 	g_return_if_fail(handle);
 
 	sself = (RsvgNodeSvg *)handle->treebase;	
-	if(!sself) {
-		memset(output, 0, sizeof(RsvgDimensionData));
+	if(!sself)
 		return;
-	}
 	
 	bbox.x = bbox.y = 0;
 	bbox.w = bbox.h = 1;
@@ -1158,17 +1088,16 @@ rsvg_handle_get_dimensions(RsvgHandle * handle, RsvgDimensionData * output)
 			else
 				bbox = _rsvg_find_bbox(handle);
 		}
-	output->width  = _rsvg_css_hand_normalize_length(&sself->w, handle->dpi_x, 
+	dimension_data->width  = _rsvg_css_hand_normalize_length(&sself->w, handle->dpi_x, 
 																	bbox.w + bbox.x * 2, 12);
-	output->height = _rsvg_css_hand_normalize_length(&sself->h, handle->dpi_y, 
+	dimension_data->height = _rsvg_css_hand_normalize_length(&sself->h, handle->dpi_y, 
 													 bbox.h + bbox.y * 2, 12);
 	
-	output->em = output->width;
-	output->ex = output->height;
+	dimension_data->em = dimension_data->width;
+	dimension_data->ex = dimension_data->height;
 
-	if (handle->size_func) {
-		(* handle->size_func) (&output->width, &output->height, handle->user_data);
-	}	
+	if (handle->size_func)
+		(* handle->size_func) (&dimension_data->width, &dimension_data->height, handle->user_data);
 }
 
 /** 
@@ -1202,14 +1131,14 @@ void
 rsvg_set_default_dpi_x_y (double dpi_x, double dpi_y)
 {
 	if (dpi_x <= 0.)
-		internal_dpi_x = RSVG_DEFAULT_DPI_X;
+		rsvg_internal_dpi_x = RSVG_DEFAULT_DPI_X;
 	else
-		internal_dpi_x = dpi_x;
+		rsvg_internal_dpi_x = dpi_x;
 
 	if (dpi_y <= 0.)
-		internal_dpi_y = RSVG_DEFAULT_DPI_Y;
+		rsvg_internal_dpi_y = RSVG_DEFAULT_DPI_Y;
 	else
-		internal_dpi_y = dpi_y;
+		rsvg_internal_dpi_y = dpi_y;
 }
 
 /**
@@ -1230,13 +1159,13 @@ rsvg_handle_set_dpi (RsvgHandle * handle, double dpi)
 }
 
 /**
- * rsvg_handle_set_dpi
+ * rsvg_handle_set_dpi_x_y
  * @handle: An #RsvgHandle
  * @dpi_x: Dots Per Inch (aka Pixels Per Inch)
  * @dpi_y: Dots Per Inch (aka Pixels Per Inch)
  *
  * Sets the DPI for the outgoing pixbuf. Common values are
- * 75, 90, and 300 DPI. Passing a number <= 0 to #dpi will 
+ * 75, 90, and 300 DPI. Passing a number <= 0 to #dpi_x or #dpi_y will 
  * reset the DPI to whatever the default value happens to be.
  *
  * Since: 2.8
@@ -1247,12 +1176,12 @@ rsvg_handle_set_dpi_x_y (RsvgHandle * handle, double dpi_x, double dpi_y)
 	g_return_if_fail (handle != NULL);
 	
     if (dpi_x <= 0.)
-        handle->dpi_x = internal_dpi_x;
+        handle->dpi_x = rsvg_internal_dpi_x;
     else
         handle->dpi_x = dpi_x;
 	
 	if (dpi_y <= 0.)
-        handle->dpi_y = internal_dpi_y;
+        handle->dpi_y = rsvg_internal_dpi_y;
     else
         handle->dpi_y = dpi_y;
 }
@@ -1269,6 +1198,8 @@ rsvg_handle_set_dpi_x_y (RsvgHandle * handle, double dpi_x, double dpi_y)
  * in to the function, which may then modify these values to set the real size
  * of the generated pixbuf.  If the image has no associated size, then the size
  * arguments are set to -1.
+ *
+ * Deprecated: Set up a cairo matrix and use rsvg_cairo_render() instead.
  **/
 void
 rsvg_handle_set_size_callback (RsvgHandle     *handle,
@@ -1398,25 +1329,6 @@ rsvg_handle_close (RsvgHandle  *handle,
 	return rsvg_handle_close_impl (handle, error);
 }
 
-/**
- * rsvg_handle_free:
- * @handle: An #RsvgHandle
- *
- * Frees #handle.
- **/
-void
-rsvg_handle_free (RsvgHandle *handle)
-{
-	g_return_if_fail(handle);
-
-#if HAVE_SVGZ
-	if (handle->is_gzipped)
-		g_object_unref (G_OBJECT (handle->gzipped_data));
-#endif
-
-	rsvg_handle_free_impl (handle);
-}
-
 #ifdef HAVE_GNOME_VFS
 #include <libgnomevfs/gnome-vfs.h>
 #endif
