@@ -67,13 +67,13 @@ rsvg_handle_new_from_stdio_file (FILE * f,
 				{
 					if (!rsvg_handle_write (handle, buffer, length, error))
 						{
-							rsvg_handle_free (handle);
+							g_object_unref (G_OBJECT(handle));
 							return NULL;
 						}
 				}
 			else if (ferror (f)) 
 				{
-					rsvg_handle_free (handle);
+					g_object_unref (G_OBJECT(handle));
 					return NULL;
 				}
 		}
@@ -86,7 +86,6 @@ rsvg_handle_new_from_stdio_file (FILE * f,
 	return handle;
 }
 
-
 static void
 rsvg_cairo_size_callback (int *width,
 						  int *height,
@@ -182,7 +181,7 @@ main (int argc, const char **argv)
 		}
 	else if (n_args > 1 && (!format || !strcmp (format, "png"))) 
 		{
-			g_print (_("Multiple SVG files are only allowed for PDF and PS output.\n"));
+			g_print (_("Multiple SVG files are only allowed for PDF, PS and SVG output.\n"));
 			exit (1);
 		}
 
@@ -255,17 +254,17 @@ main (int argc, const char **argv)
 															  dimensions.width, dimensions.height);
 #ifdef CAIRO_HAS_PDF_SURFACE
 					else if (!strcmp (format, "pdf"))
-						surface = cairo_pdf_surface_create_for_stream (rsvg_cairo_write_func, output_file, 
+						surface = cairo_pdf_surface_create_for_stream (rsvg_cairo_write_func, output_file,
 																	   dimensions.width, dimensions.height);
 #endif
 #ifdef CAIRO_HAS_PS_SURFACE
 					else if (!strcmp (format, "ps"))
-						surface = cairo_ps_surface_create_for_stream (rsvg_cairo_write_func, output_file, 
+						surface = cairo_ps_surface_create_for_stream (rsvg_cairo_write_func, output_file,
 																	  dimensions.width, dimensions.height);
 #endif
 #ifdef CAIRO_HAS_SVG_SURFACE
 					else if (!strcmp (format, "svg"))
-						surface = cairo_svg_surface_create_for_stream (rsvg_cairo_write_func, output_file, 
+						surface = cairo_svg_surface_create_for_stream (rsvg_cairo_write_func, output_file,
 																	   dimensions.width, dimensions.height);
 #endif
 					else 
@@ -284,7 +283,7 @@ main (int argc, const char **argv)
 			else
 				cairo_show_page (cr);
 
-			rsvg_handle_free (rsvg);
+			g_object_unref (G_OBJECT(rsvg));
 		}
 	
 	cairo_destroy (cr);
