@@ -145,7 +145,7 @@ rsvg_pixbuf_from_data_with_size_data (const guchar * buff,
 
 	rsvg_handle_close (handle, error);
 	retval = rsvg_handle_get_pixbuf (handle);
-	rsvg_handle_free (handle);
+	g_object_unref (G_OBJECT(handle));
 
 	return retval;	
 }
@@ -175,7 +175,7 @@ rsvg_pixbuf_from_stdio_file_with_size_data(GByteArray *f,
 
 	rsvg_handle_close (handle, error);
 	retval = rsvg_handle_get_pixbuf (handle);
-	rsvg_handle_free (handle);
+	g_object_unref (G_OBJECT(handle));
 
 	return retval;
 }
@@ -217,6 +217,7 @@ rsvg_pixbuf_from_file_with_size_data (const gchar * file_name,
  * set and %NULL is returned.
  * 
  * Return value: A newly allocated #GdkPixbuf, or %NULL
+ * Deprecated: Set up a cairo matrix and use rsvg_handle_new_from_file() + rsvg_cairo_render() instead.
  **/
 GdkPixbuf *
 rsvg_pixbuf_from_file (const gchar *file_name,
@@ -238,6 +239,7 @@ rsvg_pixbuf_from_file (const gchar *file_name,
  * occurred, @error is set and %NULL is returned.
  * 
  * Return value: A newly allocated #GdkPixbuf, or %NULL
+ * Deprecated: Set up a cairo matrix and use rsvg_handle_new_from_file() + rsvg_cairo_render() instead.
  **/
 GdkPixbuf *
 rsvg_pixbuf_from_file_at_zoom (const gchar *file_name,
@@ -274,6 +276,7 @@ rsvg_pixbuf_from_file_at_zoom (const gchar *file_name,
  * returned pixbuf. If an error occurred, @error is set and %NULL is returned.
  * 
  * Return value: A newly allocated #GdkPixbuf, or %NULL
+ * Deprecated: Set up a cairo matrix and use rsvg_handle_new_from_file() + rsvg_cairo_render() instead.
  **/
 GdkPixbuf  *
 rsvg_pixbuf_from_file_at_zoom_with_max (const gchar  *file_name,
@@ -312,6 +315,7 @@ rsvg_pixbuf_from_file_at_zoom_with_max (const gchar  *file_name,
  * error occurred, @error is set and %NULL is returned.
  * 
  * Return value: A newly allocated #GdkPixbuf, or %NULL
+ * Deprecated: Set up a cairo matrix and use rsvg_handle_new_from_file() + rsvg_cairo_render() instead.
  **/
 GdkPixbuf *
 rsvg_pixbuf_from_file_at_size (const gchar *file_name,
@@ -342,6 +346,7 @@ rsvg_pixbuf_from_file_at_size (const gchar *file_name,
  * @error is set and %NULL is returned.
  * 
  * Return value: A newly allocated #GdkPixbuf, or %NULL
+ * Deprecated: Set up a cairo matrix and use rsvg_handle_new_from_file() + rsvg_cairo_render() instead.
  **/
 GdkPixbuf  *
 rsvg_pixbuf_from_file_at_max_size (const gchar     *file_name,
