@@ -61,7 +61,7 @@ rsvg_state_init (RsvgState *state)
 	state->fill = rsvg_paint_server_parse (NULL, NULL, "#000", 0);
 	state->fill_opacity = 0xff;
 	state->stroke_opacity = 0xff;
-	state->stroke_width = 1;
+	state->stroke_width = _rsvg_css_parse_length("1");
 	state->miter_limit = 4;
 	state->cap = RSVG_PATH_STROKE_CAP_BUTT;
 	state->join = RSVG_PATH_STROKE_JOIN_MITER;
@@ -70,9 +70,11 @@ rsvg_state_init (RsvgState *state)
 	state->clip_rule = FILL_RULE_NONZERO;
 	state->backgroundnew = FALSE;	
 	state->overflow = FALSE;
+	state->flood_color = 0;
+	state->flood_opacity = 255;
 
 	state->font_family  = g_strdup (RSVG_DEFAULT_FONT);
-	state->font_size    = 12.0;
+	state->font_size    = _rsvg_css_parse_length("12.0");
 	state->font_style   = PANGO_STYLE_NORMAL;
 	state->font_variant = PANGO_VARIANT_NORMAL;
 	state->font_weight  = PANGO_WEIGHT_NORMAL;
@@ -89,6 +91,8 @@ rsvg_state_init (RsvgState *state)
 	state->endMarker    = NULL;
 
 	state->has_current_color = FALSE;
+	state->has_flood_color = FALSE;
+	state->has_flood_opacity = FALSE;
 	state->has_fill_server = FALSE;
 	state->has_fill_opacity = FALSE;
 	state->has_fill_rule = FALSE;
@@ -160,6 +164,10 @@ rsvg_state_inherit_run (RsvgState *dst, const RsvgState *src,
 	
 	if (function(dst->has_current_color, src->has_current_color))
 		dst->current_color = src->current_color;
+	if (function(dst->has_flood_color, src->has_flood_color))
+		dst->flood_color = src->flood_color;
+	if (function(dst->has_flood_opacity, src->has_flood_opacity))
+		dst->flood_opacity = src->flood_opacity;
 	if (function(dst->has_fill_server, src->has_fill_server))
 		{
 			rsvg_paint_server_ref (src->fill);
@@ -352,12 +360,15 @@ rsvg_parse_style_arg (RsvgHandle *ctx, RsvgState *state, const char *str)
 	arg_off = rsvg_css_param_arg_offset (str);
 
 	if (rsvg_css_param_match (str, "color"))
-		{
-				state->current_color = rsvg_css_parse_color (str + arg_off, &state->has_current_color);
-		}
+		state->current_color = rsvg_css_parse_color (str + arg_off, &state->has_current_color);
 	else if (rsvg_css_param_match (str, "opacity"))
+		state->opacity = rsvg_css_parse_opacity (str + arg_off);
+	else if (rsvg_css_param_match (str, "flood-color"))
+		state->flood_color = rsvg_css_parse_color (str + arg_off, &state->has_flood_color);
+	else if (rsvg_css_param_match (str, "flood-opacity"))
 		{
-			state->opacity = rsvg_css_parse_opacity (str + arg_off);
+			state->flood_opacity = rsvg_css_parse_opacity (str + arg_off);
+			state->has_flood_opacity = TRUE;
 		}
 	else if (rsvg_css_param_match (str, "filter"))
 		state->filter = rsvg_filter_parse(ctx->defs, str + arg_off);
@@ -471,8 +482,7 @@ rsvg_parse_style_arg (RsvgHandle *ctx, RsvgState *state, const char *str)
 		}
 	else if (rsvg_css_param_match (str, "stroke-width"))
 		{
-			state->stroke_width = rsvg_css_parse_normalized_length (str + arg_off, ctx->dpi_x, 
-																	(gdouble)ctx->height, state->font_size);
+			state->stroke_width = _rsvg_css_parse_length (str + arg_off);
 			state->has_stroke_width = TRUE;
 		}
 	else if (rsvg_css_param_match (str, "stroke-linecap"))
@@ -506,15 +516,8 @@ rsvg_parse_style_arg (RsvgHandle *ctx, RsvgState *state, const char *str)
 		}
 	else if (rsvg_css_param_match (str, "font-size"))
 		{
-			state->font_size =  rsvg_css_parse_normalized_length (str + arg_off, ctx->dpi_y, 
-																  (gdouble)ctx->height, state->font_size);
+			state->font_size =  _rsvg_css_parse_length (str + arg_off);
 			state->has_font_size = TRUE;
-			if (ctx != NULL)
-				{
-					((RsvgDimensionData *)g_slist_nth(ctx->dimensions, 
-													  0)->data)->em 
-						= state->font_size;
-				}
 		}
 	else if (rsvg_css_param_match (str, "font-family"))
 		{
@@ -667,10 +670,9 @@ rsvg_parse_style_arg (RsvgHandle *ctx, RsvgState *state, const char *str)
 	else if (rsvg_css_param_match (str, "stroke-dashoffset"))
 		{
 			state->has_dash = TRUE;
-			state->dash.offset = rsvg_css_parse_normalized_length (str + arg_off, rsvg_dpi_percentage (ctx), 
-																   rsvg_viewport_percentage((gdouble)ctx->width, (gdouble)ctx->height), state->font_size);
-			if (state->dash.offset < 0.)
-				state->dash.offset = 0.;
+			state->dash.offset = _rsvg_css_parse_length (str + arg_off);
+			if (state->dash.offset.length < 0.)
+				state->dash.offset.length = 0.;
 		}
 	else if (rsvg_css_param_match (str, "stroke-dasharray"))
 		{
@@ -760,6 +762,8 @@ rsvg_parse_style_pairs (RsvgHandle *ctx, RsvgState *state,
 	rsvg_lookup_parse_style_pair (ctx, state, "fill-opacity", atts);
 	rsvg_lookup_parse_style_pair (ctx, state, "fill-rule", atts);
 	rsvg_lookup_parse_style_pair (ctx, state, "filter", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "flood-color", atts);
+	rsvg_lookup_parse_style_pair (ctx, state, "flood-opacity", atts);
 	rsvg_lookup_parse_style_pair (ctx, state, "font-family", atts);
 	rsvg_lookup_parse_style_pair (ctx, state, "font-size", atts);
 	rsvg_lookup_parse_style_pair (ctx, state, "font-stretch", atts);
@@ -1360,16 +1364,6 @@ rsvg_state_parent (RsvgDrawingCtx *ctx)
 	return g_slist_nth_data(ctx->state, 1);
 }
 
-/*aweful naming of this function, needs to be fixed later*/
-
-double
-rsvg_state_current_font_size (RsvgHandle *ctx)
-{
-	if (ctx->dimensions == NULL)
-		return 12.0;
-	return ((RsvgDimensionData *)g_slist_nth(ctx->dimensions, 0)->data)->em;
-}
-
 RsvgPropertyBag *
 rsvg_property_bag_new (const xmlChar **atts)
 {
